@@ -70,11 +70,17 @@
 void
 nautilus_drag_init (NautilusDragInfo     *drag_info,
 		    const GtkTargetEntry *drag_types,
-		    int                   drag_type_count)
+		    int                   drag_type_count,
+		    gboolean              add_text_targets)
 {
 	drag_info->target_list = gtk_target_list_new (drag_types,
 						   drag_type_count);
 
+	if (add_text_targets) {
+		gtk_target_list_add_text_targets (drag_info->target_list,
+						  NAUTILUS_ICON_DND_TEXT);
+	}
+
 	drag_info->drop_occured = FALSE;
 	drag_info->need_to_destroy = FALSE;
 }
@@ -280,6 +286,22 @@ nautilus_drag_items_on_desktop (const GList *selection_list)
 	
 }
 
+GdkDragAction
+nautilus_drag_default_drop_action_for_url (GdkDragContext *context)
+{
+	/* Mozilla defaults to copy, but unless thats the
+	   only allowed thing (enforced by ctrl) we want to ASK */
+	if (context->suggested_action == GDK_ACTION_COPY &&
+	    context->actions != GDK_ACTION_COPY) {
+		return GDK_ACTION_ASK;
+	} else if (context->suggested_action == GDK_ACTION_MOVE) {
+		/* Don't support move */
+		return GDK_ACTION_COPY;
+	}
+	
+	return context->suggested_action;
+}
+
 
 void
 nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
@@ -371,7 +393,7 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 			*action = context->suggested_action;
 		}
 	} else {
-		if (actions & GDK_ACTION_MOVE) {
+		if (actions & GDK_ACTION_COPY) {
 			*action = GDK_ACTION_COPY;
 		} else {
 			*action = context->suggested_action;
@@ -683,8 +705,8 @@ nautilus_drag_drop_background_ask (GtkWidget *widget,
 				      &damd);
 
 	append_drop_action_menu_item (menu, _("Set as background for _this folder"),
-				      NAUTILUS_DND_ACTION_SET_AS_BACKGROUND,
-				      (actions & NAUTILUS_DND_ACTION_SET_AS_BACKGROUND) != 0,
+				      NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND,
+				      (actions & NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND) != 0,
 				      &damd);
 
 	menu_item = gtk_separator_menu_item_new ();
