@@ -26,6 +26,7 @@
 
 #include <config.h>
 #include <string.h>
+#include <stdio.h>
 #include "nautilus-file-operations.h"
 
 #include "nautilus-file-operations-progress.h"
@@ -38,6 +39,7 @@
 #include <eel/eel-vfs-extensions.h>
 
 #include <gnome.h>
+#include <gdk/gdkdnd.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmessagedialog.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
@@ -326,7 +328,9 @@ progress_dialog_set_to_from_item_text (NautilusFileOperationsProgress *dialog,
 {
 	char *item;
 	char *from_path;
+	char *from_text;
 	char *to_path;
+	char *to_text;
 	char *progress_label_text;
 	const char *from_prefix;
 	const char *to_prefix;
@@ -334,8 +338,8 @@ progress_dialog_set_to_from_item_text (NautilusFileOperationsProgress *dialog,
 	int length;
 
 	item = NULL;
-	from_path = NULL;
-	to_path = NULL;
+	from_text = NULL;
+	to_text = NULL;
 	from_prefix = "";
 	to_prefix = "";
 	progress_label_text = NULL;
@@ -350,10 +354,18 @@ progress_dialog_set_to_from_item_text (NautilusFileOperationsProgress *dialog,
 		if (from_path [length - 1] == '/') {
 			from_path [length - 1] = '\0';
 		}
+
+		if (g_str_has_prefix (from_uri, "file://")) {
+			from_text = from_path;
+		} else {
+			from_text = g_strdup_printf (_("%s on %s"),
+				from_path, gnome_vfs_uri_get_host_name (uri));
+			g_free (from_path);
+		}
 		
 		gnome_vfs_uri_unref (uri);
 		g_assert (progress_verb);
-		progress_label_text = g_strdup_printf ("%s:", progress_verb);
+		progress_label_text = g_strdup_printf ("%s", progress_verb);
 		/* "From" dialog label, source path gets placed next to it in the dialog */
 		from_prefix = _("From:");
 	}
@@ -368,6 +380,14 @@ progress_dialog_set_to_from_item_text (NautilusFileOperationsProgress *dialog,
 			to_path [length - 1] = '\0';
 		}
 
+		if (g_str_has_prefix (to_uri, "file://")) {
+			to_text = to_path;
+		} else {
+			to_text = g_strdup_printf (_("%s on %s"),
+				to_path, gnome_vfs_uri_get_host_name (uri));
+			g_free (to_path);
+		}
+
 		gnome_vfs_uri_unref (uri);
 		/* "To" dialog label, source path gets placed next to it in the dialog */
 		to_prefix = _("To:");
@@ -377,14 +397,14 @@ progress_dialog_set_to_from_item_text (NautilusFileOperationsProgress *dialog,
 		(dialog,
 		 progress_label_text != NULL ? progress_label_text : "",
 		 item != NULL ? item : "",
-		 from_path != NULL ? from_path : "",
-		 to_path != NULL ? to_path : "",
+		 from_text != NULL ? from_text : "",
+		 to_text != NULL ? to_text : "",
 		 from_prefix, to_prefix, index, size);
 
 	g_free (progress_label_text);
 	g_free (item);
-	g_free (from_path);
-	g_free (to_path);
+	g_free (from_text);
+	g_free (to_text);
 }
 
 static int
@@ -932,36 +952,25 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 				TRUE, GTK_MESSAGE_ERROR, text, detail, dialog_title, GTK_STOCK_OK, NULL);
 			error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 
-		} else if ((error_location == ERROR_LOCATION_SOURCE
-				|| error_location == ERROR_LOCATION_SOURCE_PARENT
-				|| error_location == ERROR_LOCATION_SOURCE_OR_PARENT)
-			&& (error_kind == ERROR_NOT_ENOUGH_PERMISSIONS
-				|| error_kind == ERROR_NOT_READABLE)) {
-			/* The error could have happened on any of the files
-			 * in the moved/copied/deleted hierarchy, we can probably
-			 * continue. Allow the user to skip.
-			 */
+		} else if (progress_info->files_total == 1) {
 			error_dialog_button_pressed = eel_run_simple_dialog
 				(parent_for_error_dialog (transfer_info), TRUE, 
 				 GTK_MESSAGE_ERROR, text, 
 				 detail, dialog_title,
-				 GTK_STOCK_CANCEL, _("_Skip"), NULL);
-				 
+				 GTK_STOCK_CANCEL, _("_Retry"), NULL);
+
 			switch (error_dialog_button_pressed) {
 			case 0:
-			case GTK_RESPONSE_DELETE_EVENT:
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 				break;
 			case 1:
-				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_SKIP;
+				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_RETRY;
 				break;
 			default:
 				g_assert_not_reached ();
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 			}
-								
 		} else {
-			/* Generic error, offer to retry and skip. */
 			error_dialog_button_pressed = eel_run_simple_dialog
 				(parent_for_error_dialog (transfer_info), TRUE, 
 				 GTK_MESSAGE_ERROR, text, 
@@ -983,7 +992,7 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 			}
 		}
-			
+
 		g_free (text);
 		g_free (detail);
 		g_free (formatted_source_name);
@@ -1338,7 +1347,7 @@ parse_previous_duplicate_name (const char *name,
 
 	g_assert (name[0] != '\0');
 	
-	*suffix = strrchr (name + 1, '.');
+	*suffix = strchr (name + 1, '.');
 	if (*suffix == NULL || (*suffix)[1] == '\0') {
 		/* no suffix */
 		*suffix = "";
@@ -1852,9 +1861,10 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		} else {
 			/* duplication */
 			target_uri = gnome_vfs_uri_ref (source_uri);
-			if (target_dir_uri == NULL) {
-				target_dir_uri = gnome_vfs_uri_ref (source_dir_uri);
+			if (target_dir_uri != NULL) {
+				gnome_vfs_uri_unref (target_dir_uri);
 			}
+			target_dir_uri = gnome_vfs_uri_ref (source_dir_uri);
 		}
 		
 		if (target_uri != NULL) {
@@ -2241,7 +2251,6 @@ typedef struct {
 	NautilusNewFileCallback done_callback;
 	gpointer data;
 	GtkWidget *parent_view;
-	char *empty_file;
 	GHashTable *debuting_uris;
 } NewFileTransferState;
 
@@ -2294,6 +2303,7 @@ new_file_transfer_callback (GnomeVFSAsyncHandle *handle,
 {
 	NewFileTransferState *state;
 	char *temp_string;
+	char **temp_strings;
 	char *uri;
 	
 	state = (NewFileTransferState *) data;
@@ -2308,11 +2318,6 @@ new_file_transfer_callback (GnomeVFSAsyncHandle *handle,
 
 		(* state->done_callback) (uri, state->data);
 		/* uri is owned by hashtable, don't free */
-
-		if (state->empty_file != NULL) {
-			unlink (state->empty_file);
-			g_free (state->empty_file);
-		}
 		eel_remove_weak_pointer (&state->parent_view);
 		g_hash_table_destroy (state->debuting_uris);
 		g_free (state);
@@ -2339,10 +2344,20 @@ new_file_transfer_callback (GnomeVFSAsyncHandle *handle,
 					 progress_info->duplicate_name,
 					 progress_info->duplicate_count);
 			} else {
-				progress_info->duplicate_name = g_strdup_printf
-					("%s%%20%d", 
-					 progress_info->duplicate_name,
-					 progress_info->duplicate_count);
+				temp_strings = g_strsplit (temp_string, ".", 2);
+				if (temp_strings[1] != NULL) {
+					progress_info->duplicate_name = g_strdup_printf
+						("%s%%20%d.%s", 
+						 temp_strings[0],
+						 progress_info->duplicate_count,
+						 temp_strings[1]);
+				} else {
+					progress_info->duplicate_name = g_strdup_printf
+						("%s%%20%d", 
+						 progress_info->duplicate_name,
+						 progress_info->duplicate_count);
+				}
+				g_strfreev (temp_strings);
 			}
 			g_free (temp_string);
 			return GNOME_VFS_XFER_ERROR_ACTION_SKIP;
@@ -2361,61 +2376,48 @@ new_file_transfer_callback (GnomeVFSAsyncHandle *handle,
 }
 
 void 
-nautilus_file_operations_new_file (GtkWidget *parent_view, 
-				   const char *parent_dir,
-				   const char *source_uri_text,
-				   NautilusNewFileCallback done_callback,
-				   gpointer data)
+nautilus_file_operations_new_file_from_template (GtkWidget *parent_view, 
+						 const char *parent_dir,
+						 const char *target_filename,
+						 const char *template_uri,
+						 gboolean move_template,
+						 NautilusNewFileCallback done_callback,
+						 gpointer data)
 {
 	GList *target_uri_list;
 	GList *source_uri_list;
 	GnomeVFSURI *target_uri, *parent_uri, *source_uri;
-	char *filename;
+	GnomeVFSXferOptions options;
 	NewFileTransferState *state;
 	SyncTransferInfo *sync_transfer_info;
+	char *tmp;
+
+	g_assert (parent_dir != NULL);
+	g_assert (template_uri != NULL);
 
 	state = g_new (NewFileTransferState, 1);
 	state->done_callback = done_callback;
 	state->data = data;
 	state->parent_view = parent_view;
-	state->empty_file = NULL;
 
 	/* pass in the target directory and the new folder name as a destination URI */
 	parent_uri = gnome_vfs_uri_new (parent_dir);
 
-	if (source_uri_text != NULL) {
-		source_uri = gnome_vfs_uri_new (source_uri_text);
-		if (source_uri == NULL) {
-			(*done_callback) (NULL, data);
-			g_free (state);
-			return;
-		}
-		filename = gnome_vfs_uri_extract_short_path_name (source_uri);
-		target_uri = gnome_vfs_uri_append_string (parent_uri, filename);
-		g_free (filename);
-	} else {
-		char empty_file[] = "/tmp/emptyXXXXXX";
-		char *empty_uri;
-		int fd;
-
-		fd = mkstemp (empty_file);
-		if (fd == -1) {
-			(*done_callback) (NULL, data);
-			g_free (state);
-		}
-		close (fd);
-
-		empty_uri = gnome_vfs_get_uri_from_local_path (empty_file);
-		source_uri = gnome_vfs_uri_new (empty_uri);
-		g_free (empty_uri);
-		
-		state->empty_file = g_strdup (empty_file);
-		
-		filename = g_filename_from_utf8 (_("new file"), -1, NULL, NULL, NULL);
-		target_uri = gnome_vfs_uri_append_file_name (parent_uri, filename);
-		g_free (filename);
+	source_uri = gnome_vfs_uri_new (template_uri);
+	if (source_uri == NULL) {
+		(*done_callback) (NULL, data);
+		g_free (state);
+		return;
 	}
-	
+
+	if (target_filename != NULL) {
+		target_uri = gnome_vfs_uri_append_file_name (parent_uri, target_filename);
+	} else {
+		tmp = gnome_vfs_uri_extract_short_name (source_uri);
+		target_uri = gnome_vfs_uri_append_file_name (parent_uri, tmp);
+		g_free (tmp);
+	}
+
 	state->debuting_uris = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	eel_add_weak_pointer (&state->parent_view);
 
@@ -2425,9 +2427,14 @@ nautilus_file_operations_new_file (GtkWidget *parent_view,
 	sync_transfer_info = g_new (SyncTransferInfo, 1);
 	sync_transfer_info->iterator = NULL;
 	sync_transfer_info->debuting_uris = state->debuting_uris;
-	
+
+	options = GNOME_VFS_XFER_USE_UNIQUE_NAMES;
+	if (move_template) {
+		options |= GNOME_VFS_XFER_REMOVESOURCE;
+	}
+
 	gnome_vfs_async_xfer (&state->handle, source_uri_list, target_uri_list,
-	      		      GNOME_VFS_XFER_USE_UNIQUE_NAMES,
+	      		      options,
 	      		      GNOME_VFS_XFER_ERROR_MODE_QUERY, 
 	      		      GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
 			      GNOME_VFS_PRIORITY_DEFAULT,
@@ -2439,6 +2446,50 @@ nautilus_file_operations_new_file (GtkWidget *parent_view,
 	gnome_vfs_uri_unref (parent_uri);
 }
 
+void 
+nautilus_file_operations_new_file (GtkWidget *parent_view, 
+				   const char *parent_dir,
+				   const char *initial_contents,
+				   NautilusNewFileCallback done_callback,
+				   gpointer data)
+{
+	char source_file_str[] = "/tmp/nautilus-sourceXXXXXX";
+	char *source_file_uri;
+	FILE *source_file;
+	char *target_filename;
+	int fd;
+
+	fd = mkstemp (source_file_str);
+	if (fd == -1) {
+		(*done_callback) (NULL, data);
+		return;
+	}
+
+	if (initial_contents != NULL) {
+		source_file = fdopen (fd, "a+");
+
+		fprintf (source_file, "%s", initial_contents);
+		fclose (source_file);
+	}
+
+	close (fd);
+
+	target_filename = g_filename_from_utf8 (_("new file"), -1, NULL, NULL, NULL);
+
+	source_file_uri = gnome_vfs_get_uri_from_local_path (source_file_str);
+
+	nautilus_file_operations_new_file_from_template (parent_view, 
+							 parent_dir,
+							 target_filename,
+							 source_file_uri,
+							 TRUE,
+							 done_callback,
+							 data);
+
+	g_free (source_file_uri);
+	g_free (target_filename);
+}
+
 void 
 nautilus_file_operations_delete (const GList *item_uris, 
 				 GtkWidget *parent_view)
@@ -2638,8 +2689,8 @@ nautilus_self_check_file_operations (void)
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo.txt", 1), "foo (copy).txt");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo foo.txt", 1), "foo foo (copy).txt");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo foo.txt txt", 1), "foo foo (copy).txt txt");
-	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...txt", 1), "foo.. (copy).txt");
-	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...", 1), "foo... (copy)");
+	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...txt", 1), "foo (copy)...txt");
+	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...", 1), "foo (copy)...");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo. (copy)", 1), "foo. (another copy)");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (copy)", 1), "foo (another copy)");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (copy).txt", 1), "foo (another copy).txt");
