@@ -50,7 +50,6 @@
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-metadata.h>
-#include <libnautilus-private/nautilus-theme.h>
 #include <libnautilus-private/nautilus-sidebar.h>
 #include <string.h>
 #include <stdlib.h>
@@ -59,6 +58,7 @@
 #define MAX_TITLE_SIZE 		256
 #define MINIMUM_INFO_WIDTH	32
 #define SIDEBAR_INFO_MARGIN	4
+#define SHADOW_OFFSET		1
 
 #define MORE_INFO_FONT_SIZE 	 12
 #define MIN_TITLE_FONT_SIZE 	 12
@@ -69,11 +69,11 @@ static void                nautilus_sidebar_title_destroy          (GtkObject
 static void                nautilus_sidebar_title_init       (NautilusSidebarTitle      *pixmap);
 static void                nautilus_sidebar_title_size_allocate    (GtkWidget                 *widget,
 								    GtkAllocation             *allocation);
-static void                nautilus_sidebar_title_theme_changed    (gpointer                   user_data);
 static void                update_icon                             (NautilusSidebarTitle      *sidebar_title);
 static GtkWidget *         sidebar_title_create_title_label        (void);
 static GtkWidget *         sidebar_title_create_more_info_label    (void);
 static void		   update_all 				   (NautilusSidebarTitle      *sidebar_title);
+static void		   update_more_info			   (NautilusSidebarTitle      *sidebar_title);
 static void		   update_title_font			   (NautilusSidebarTitle      *sidebar_title);
 static void                style_set                               (GtkWidget                 *widget,
 								    GtkStyle                  *previous_style);
@@ -90,7 +90,8 @@ struct NautilusSidebarTitleDetails {
 	GtkWidget		*emblem_box;
 	GtkWidget		*notes;
 
-	int			 shadow_offset;
+	guint                    best_icon_size;
+
 	gboolean		 determined_icon;
 };
 
@@ -174,14 +175,13 @@ nautilus_sidebar_title_init (NautilusSidebarTitle *sidebar_title)
 	/* Keep track of changes in graphics trade offs */
 	update_all (sidebar_title);
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_THEME,
-						  nautilus_sidebar_title_theme_changed,
-						  sidebar_title,
-						  G_OBJECT (sidebar_title));
-
 	/* initialize the label colors & fonts */
-	nautilus_sidebar_title_theme_changed (sidebar_title);
 	style_set (GTK_WIDGET (sidebar_title), NULL);
+
+	eel_preferences_add_callback_while_alive (
+		NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+		(EelPreferencesCallback) update_more_info,
+		sidebar_title, G_OBJECT (sidebar_title));
 }
 
 /* destroy by throwing away private storage */
@@ -245,32 +245,16 @@ nautilus_sidebar_title_select_text_color (NautilusSidebarTitle *sidebar_title,
 	sidebar_info_title_color = NULL;
 	sidebar_title_shadow_color = NULL;
 	
-	if (is_default) {
-		sidebar_title_color = nautilus_theme_get_theme_data ("sidebar", "title_color");
-		sidebar_info_title_color = nautilus_theme_get_theme_data ("sidebar", "title_info_color");
-		sidebar_title_shadow_color = nautilus_theme_get_theme_data ("sidebar", "title_shadow_color"); 
-	}
-		
-	if (sidebar_title_color == NULL) {
-		/* FIXME bugzilla.gnome.org 42496: for now, both the title and info
-		 * colors are the same - and hard coded */
-		if (eel_background_is_dark (background)) {
-			sidebar_title_color = g_strdup ("#FFFFFF");
-			sidebar_info_title_color = g_strdup ("#FFFFFF");
-			sidebar_title_shadow_color = g_strdup ("#000000");
-				
-		} else {
-			sidebar_title_color = g_strdup ("#000000");
-			sidebar_info_title_color = g_strdup ("#000000");
-			sidebar_title_shadow_color = g_strdup ("#FFFFFF");
-		}
+	/* FIXME bugzilla.gnome.org 42496: for now, both the title and info
+	 * colors are the same - and hard coded */
+	if (eel_background_is_dark (background)) {
+		sidebar_title_color = g_strdup ("#FFFFFF");
+		sidebar_info_title_color = g_strdup ("#FFFFFF");
+		sidebar_title_shadow_color = g_strdup ("#000000");
 	} else {
-		if (sidebar_info_title_color == NULL) {
-			sidebar_info_title_color = g_strdup (sidebar_title_color);
-		}
-		if (sidebar_title_shadow_color == NULL) {
-			sidebar_title_shadow_color = g_strdup ("#FFFFFF");
-		}
+		sidebar_title_color = g_strdup ("#000000");
+		sidebar_info_title_color = g_strdup ("#000000");
+		sidebar_title_shadow_color = g_strdup ("#FFFFFF");
 	}
 
 	eel_gtk_widget_set_foreground_color (sidebar_title->details->title_label,
@@ -283,34 +267,16 @@ nautilus_sidebar_title_select_text_color (NautilusSidebarTitle *sidebar_title,
 	eel_gtk_label_set_drop_shadow_color (GTK_LABEL (sidebar_title->details->more_info_label),
 					     eel_parse_rgb_with_white_default (sidebar_title_shadow_color));
 
-	eel_gtk_label_set_drop_shadow_offset (GTK_LABEL (sidebar_title->details->title_label),
-					      sidebar_title->details->shadow_offset);
+	eel_gtk_label_set_drop_shadow_offset (GTK_LABEL (sidebar_title->details->title_label), 
+					      SHADOW_OFFSET);
 	eel_gtk_label_set_drop_shadow_offset (GTK_LABEL (sidebar_title->details->more_info_label),
-					      sidebar_title->details->shadow_offset);
+					      SHADOW_OFFSET);
 		
 	g_free (sidebar_title_color);	
 	g_free (sidebar_info_title_color);	
 	g_free (sidebar_title_shadow_color);
 }
 
-/* handle theme changes by setting up the color of the labels */
-static void
-nautilus_sidebar_title_theme_changed (gpointer user_data)
-{
-	char *shadow_offset_str;
-	NautilusSidebarTitle *sidebar_title;	
-
-	sidebar_title = NAUTILUS_SIDEBAR_TITLE (user_data);		
-	
-	shadow_offset_str = nautilus_theme_get_theme_data ("sidebar", "shadow_offset");
-	if (shadow_offset_str) {
-		sidebar_title->details->shadow_offset = atoi (shadow_offset_str);	
-		g_free (shadow_offset_str);
-	} else {
-		sidebar_title->details->shadow_offset = 1;	
-	}
-}
-
 static char*
 get_property_from_component (NautilusSidebarTitle *sidebar_title, const char *property)
 {
@@ -320,6 +286,21 @@ get_property_from_component (NautilusSidebarTitle *sidebar_title, const char *pr
 	return NULL;
 }
 
+static guint
+get_best_icon_size (NautilusSidebarTitle *sidebar_title)
+{
+	gint width;
+
+	width = GTK_WIDGET (sidebar_title)->allocation.width - TITLE_PADDING;
+
+	if (width < 0) {
+		/* use smallest available icon size */
+		return nautilus_icon_factory_get_smaller_icon_size (0);
+	} else {
+		return nautilus_icon_factory_get_smaller_icon_size ((guint) width);
+	}
+}
+
 /* set up the icon image */
 static void
 update_icon (NautilusSidebarTitle *sidebar_title)
@@ -335,11 +316,11 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 
 	pixbuf = NULL;
 	if (icon_name != NULL && icon_name[0] != '\0') {
-		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL, NAUTILUS_ICON_SIZE_LARGE, NULL);
+		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL, NAUTILUS_ICON_SIZE_LARGE, TRUE, NULL);
 	} else if (nautilus_icon_factory_is_icon_ready_for_file (sidebar_title->details->file)) {
 		pixbuf = nautilus_icon_factory_get_pixbuf_for_file (sidebar_title->details->file,
 								    "accept",
-								    NAUTILUS_ICON_SIZE_LARGE);
+								    sidebar_title->details->best_icon_size);
 	} else if (sidebar_title->details->determined_icon) {
 		/* We used to know the icon for this file, but now the file says it isn't
 		 * ready. This means that some file info has been invalidated, which
@@ -658,16 +639,26 @@ static void
 nautilus_sidebar_title_size_allocate (GtkWidget *widget,
 				      GtkAllocation *allocation)
 {
+	NautilusSidebarTitle *sidebar_title;
 	guint16 old_width;
+	guint best_icon_size;
+
+	sidebar_title = NAUTILUS_SIDEBAR_TITLE (widget);
 
 	old_width = widget->allocation.width;
 
 	EEL_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
 
 	if (old_width != widget->allocation.width) {
+		best_icon_size = get_best_icon_size (sidebar_title);
+		if (best_icon_size != sidebar_title->details->best_icon_size) {
+			sidebar_title->details->best_icon_size = best_icon_size;
+			update_icon (sidebar_title);
+		}
+
 		/* update the title font and info format as the size changes. */
-		update_title_font (NAUTILUS_SIDEBAR_TITLE (widget));
-		update_more_info (NAUTILUS_SIDEBAR_TITLE (widget));	
+		update_title_font (sidebar_title);
+		update_more_info (sidebar_title);	
 	}
 }
 
@@ -688,6 +679,9 @@ sidebar_title_create_title_label (void)
 	eel_gtk_label_make_bold (GTK_LABEL (title_label));
 	gtk_label_set_line_wrap (GTK_LABEL (title_label), TRUE);
 	gtk_label_set_justify (GTK_LABEL (title_label), GTK_JUSTIFY_CENTER);
+	gtk_label_set_selectable (GTK_LABEL (title_label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (title_label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (title_label), 0., 0.5);
 
 	return title_label;
 }
@@ -700,6 +694,9 @@ sidebar_title_create_more_info_label (void)
 	more_info_label = gtk_label_new ("");
 	eel_gtk_label_set_scale (GTK_LABEL (more_info_label), PANGO_SCALE_SMALL);
 	gtk_label_set_justify (GTK_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
+	gtk_label_set_selectable (GTK_LABEL (more_info_label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (more_info_label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (more_info_label), 0., 0.5);
 	
 	return more_info_label;
 }
