@@ -32,11 +32,13 @@
 
 #include "nautilus-actions.h"
 #include "nautilus-application.h"
+#include "nautilus-bookmark-list.h"
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-information-panel.h"
 #include "nautilus-main.h"
 #include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
+#include "nautilus-window-bookmarks.h"
 #include "nautilus-zoom-control.h"
 #include <eel/eel-debug.h>
 #include <eel/eel-marshal.h>
@@ -156,10 +158,10 @@ nautilus_window_init (NautilusWindow *window)
 	window->details->statusbar = statusbar;
 	gtk_table_attach (GTK_TABLE (table),
 			  statusbar,
-			  /* X direction */       /* Y direction */
-			  0, 1,                   4, 5,
-			  GTK_EXPAND | GTK_FILL,  0,
-			  0,                      0);
+			  /* X direction */                   /* Y direction */
+			  0, 1,                               4, 5,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
+			  0,                                  0);
 	window->details->help_message_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR (statusbar), "help_message");
 	gtk_widget_show (statusbar);
@@ -171,10 +173,10 @@ nautilus_window_init (NautilusWindow *window)
 	gtk_widget_show (menu);
 	gtk_table_attach (GTK_TABLE (table),
 			  menu, 
-			  /* X direction */          /* Y direction */
-			  0, 1,                      0, 1,
-			  GTK_EXPAND | GTK_FILL,     0,
-			  0,                         0);
+			  /* X direction */                   /* Y direction */
+			  0, 1,                               0, 1,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
+			  0,                                  0);
 
 	
 	/* Register IconFactory callback to update the window border icon
@@ -297,6 +299,20 @@ nautilus_window_allow_up (NautilusWindow *window, gboolean allow)
 			 set_allow_up, (window, allow));
 }
 
+static void
+update_cursor (NautilusWindow *window)
+{
+	GdkCursor *cursor;
+
+	if (window->details->allow_stop) {
+		cursor = gdk_cursor_new (GDK_WATCH);
+		gdk_window_set_cursor (GTK_WIDGET (window)->window, cursor);
+		gdk_cursor_unref (cursor);
+	} else {
+		gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+	}
+}
+
 void
 nautilus_window_allow_stop (NautilusWindow *window, gboolean allow)
 {
@@ -307,6 +323,14 @@ nautilus_window_allow_stop (NautilusWindow *window, gboolean allow)
 	action = gtk_action_group_get_action (window->details->main_action_group,
 					      NAUTILUS_ACTION_STOP);
 	gtk_action_set_sensitive (action, allow);
+
+	if (window->details->allow_stop != allow) {
+		window->details->allow_stop = allow;
+
+		if (GTK_WIDGET_REALIZED (GTK_WIDGET (window))) {
+			update_cursor (window);
+		}
+	}
 	
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
 			 set_throbber_active, (window, allow));
@@ -535,6 +559,8 @@ nautilus_window_finalize (GObject *object)
 	
 	window = NAUTILUS_WINDOW (object);
 
+	nautilus_window_remove_bookmarks_menu_callback (window);
+	
 	nautilus_window_manage_views_finalize (window);
 
 	nautilus_window_set_viewed_file (window, NULL);
@@ -637,6 +663,12 @@ nautilus_window_size_request (GtkWidget		*widget,
 	}
 }
 
+static void
+nautilus_window_realize (GtkWidget *widget)
+{
+	GTK_WIDGET_CLASS (nautilus_window_parent_class)->realize (widget);
+	update_cursor (NAUTILUS_WINDOW (widget));
+}
 
 /*
  * Main API
@@ -675,6 +707,10 @@ add_view_as_menu_item (NautilusWindow *window,
 	char *action_name;
 	ActivateViewData *data;
 
+	char *accel;
+	char *accel_path;
+	unsigned int accel_keyval;
+
 	info = nautilus_view_factory_lookup (identifier);
 	
 	action_name = g_strdup_printf ("view_as_%d", index);
@@ -684,6 +720,21 @@ add_view_as_menu_item (NautilusWindow *window,
 				       NULL,
 				       0);
 
+	if (index >= 1 && index <= 9) {
+		accel = g_strdup_printf ("%d", index);
+
+		accel_path = g_strdup_printf ("<Nautilus-Window>/%s", action_name);
+
+		accel_keyval = gdk_keyval_from_name (accel);
+		g_assert (accel_keyval != GDK_VoidSymbol);
+
+		gtk_accel_map_add_entry (accel_path, accel_keyval, GDK_CONTROL_MASK);
+		gtk_action_set_accel_path (GTK_ACTION (action), accel_path);
+
+		g_free (accel);
+		g_free (accel_path);
+	}
+
 	if (window->details->view_as_radio_action != NULL) {
 		gtk_radio_action_set_group (action,
 					    gtk_radio_action_get_group (window->details->view_as_radio_action));
@@ -959,39 +1010,6 @@ nautilus_window_display_error (NautilusWindow *window, const char *error_msg)
 	gtk_widget_show (dialog);
 }
 
-static char *
-compute_default_title (const char *text_uri)
-{
-	NautilusFile *file;
-	GnomeVFSURI *uri;
-	char *title, *displayname;
-	const char *hostname;
-
-	hostname = NULL;
-
-	if (text_uri) {
-		file = nautilus_file_get (text_uri);
-		uri = gnome_vfs_uri_new (text_uri);
-		if (uri && !gnome_vfs_uri_is_local (uri)) {
-			hostname = gnome_vfs_uri_get_host_name (uri);
-		}
-		displayname = nautilus_file_get_display_name (file);
-		if (hostname) {
-			title = g_strdup_printf (_("%s on %s"), displayname, hostname);
-			g_free (displayname);
-		} else {
-			title = displayname;
-		}
-		if (uri) {
-			gnome_vfs_uri_unref (uri);
-		}
-		nautilus_file_unref (file);
-	} else {
-		title = g_strdup ("");
-	}
-	
-	return title;
-}
 
 static char *
 real_get_title (NautilusWindow *window)
@@ -1007,7 +1025,7 @@ real_get_title (NautilusWindow *window)
         }
         
 	if (title == NULL) {
-                title = compute_default_title (window->details->location);
+                title = nautilus_compute_title_for_uri (window->details->location);
         }
 
 	return title;
@@ -1091,15 +1109,10 @@ nautilus_window_update_title (NautilusWindow *window)
 void
 nautilus_window_update_icon (NautilusWindow *window)
 {
-	char *icon_name = NULL;
+	char *icon_name;
 
-	/* Desktop window special icon */
-	if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-		icon_name = g_strdup ("gnome-fs-desktop");
-	} else {
-		icon_name = nautilus_icon_factory_get_icon_for_file (window->details->viewed_file,
-								     FALSE);
-	}
+	icon_name = EEL_CALL_METHOD_WITH_RETURN_VALUE (NAUTILUS_WINDOW_CLASS, window,
+						       get_icon_name, (window));
 
 	if (icon_name != NULL) {
 		gtk_window_set_icon_name (GTK_WINDOW (window), icon_name);
@@ -1415,6 +1428,12 @@ nautilus_window_set_hidden_files_mode (NautilusWindowInfo *window,
 			       mode);
 }
 
+static NautilusBookmarkList *
+nautilus_window_get_bookmark_list (NautilusWindow *window)
+{
+  return nautilus_get_bookmark_list ();
+}
+
 static void
 nautilus_window_info_iface_init (NautilusWindowInfoIface *iface)
 {
@@ -1429,6 +1448,7 @@ nautilus_window_info_iface_init (NautilusWindowInfoIface *iface)
 	iface->get_window_type = nautilus_window_get_window_type;
 	iface->get_title = nautilus_window_get_cached_title;
 	iface->get_history = nautilus_window_get_history;
+	iface->get_bookmark_list = nautilus_window_get_bookmark_list;
 	iface->get_current_location = nautilus_window_get_location;
 	iface->get_ui_manager = nautilus_window_get_ui_manager;
 	iface->get_selection_count = nautilus_window_get_selection_count;
@@ -1448,6 +1468,7 @@ nautilus_window_class_init (NautilusWindowClass *class)
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_window_destroy;
 	GTK_WIDGET_CLASS (class)->show = nautilus_window_show;
 	GTK_WIDGET_CLASS (class)->size_request = nautilus_window_size_request;
+	GTK_WIDGET_CLASS (class)->realize = nautilus_window_realize;
 	class->add_current_location_to_history_list = real_add_current_location_to_history_list;
 	class->get_title = real_get_title;
 	class->set_title = real_set_title;
